unit CompColl;

{$DEFINE NOTRIAL}

interface

uses Messages, {$IFDEF WIN32} Windows, {$ELSE} WinTypes, WinProcs, {$ENDIF} SysUtils,
     Classes, Controls, Forms, Menus, Graphics, Tabs, Dialogs, Buttons, StdCtrls, DsgnIntf;

const  CM_COLLECT  = CM_BASE+1910;
       CM_AUTOHIDE = CM_BASE+1911;

type
  TAttachSide = (asLeft, asTop, asNone);

  TComponentCollector = class(TCustomControl)
  private
    FNVCs: TList;
    FAutoHide:Boolean;
    FAttachSide:TAttachSide;
    FHidden: Boolean;
    FTimerID:LongInt;
    FHiddenBarWidth:Integer;
    FShowTitles:Boolean;
    FShowPicture:Boolean;
    FButtonLayout:TButtonLayout;
    FSelectedEditor:TComponentEditor;
    FPopup:TPopupMenu;
    FButtonWidth:Integer;
    FButtonHeight:Integer;
    FExcludeList:TStringList;
    FTabSet:TTabSet;
    FDesigning:Boolean;
    FDForm:TForm;
    FDesigner:TFormDesigner;
    FTextTag:string;
    FVersionInfo:string;
    FMargin:Integer;
    FSpacing:Integer;
    FScrollTime:Integer;
    procedure FreeNVCs;
    procedure SetAttachSide(Value:TAttachSide);
    procedure SetAutoHide(Value:Boolean);
    procedure SetShowTitles(Value:Boolean);
    procedure SetShowPicture(Value:Boolean);
    procedure SetButtonLayout(Value:TButtonLayout);
    procedure SetButtonWidth(Value:Integer);
    procedure SetButtonHeight(Value:Integer);
    procedure SetSpacing(Value:Integer);
    procedure SetMargin(Value:Integer);
    procedure ShowComponents(Value:Boolean);
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure TabMouseMove(Sender:TObject; Shift: TShiftState; X, Y: Integer);
    function GetBarHeight:Integer;
    function GetBarWidth:Integer;
    procedure HideBar;
    procedure ShowBar;
    procedure ScrollBar(Points:Integer);
    procedure WMTimer(var Msg: TWMTimer); message WM_TIMER;
    procedure WMCreate(var Msg:TWMCreate); message WM_CREATE;
    procedure WMDestroy(var Msg:TWMDestroy); message WM_DESTROY;
    procedure CMCollect(var Msg:TMessage); message CM_COLLECT;
    procedure CMAutoHide(var Msg:TMessage); message CM_AUTOHIDE;
    function GetPaletteBitmap(AComp:TComponent):THandle;
  protected
    property Align;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Cursor;
    property ExcludeList:TStringList read FExcludeList write FExcludeList;
    procedure Paint; override;
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Collect;
    procedure DefineProperties(Filer: TFiler); override;
    procedure ReadData(Reader: TReader);
    procedure WriteData(Writer: TWriter);
    function CanWeCollect(AComp:TComponent):Boolean;
    procedure BtnClick(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure BtnDblClick(Sender:TObject);
    procedure Loaded; override;
    procedure PopupVerbMenuClick(Sender: TObject);
    procedure FreeNVC(NVCPointer:Pointer);
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure InsertGroupList(PopItem: TMenuItem);
    function GetButtonHeight:Integer;
  public
    property Enabled;
    property Visible;
    property Designer:TFormDesigner read FDesigner write FDesigner;
    property DForm:TForm read FDForm write FDForm;
    property NVCs:TList read FNVCs write FNVCs;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure AddComponent(Name, Title, ClassName:string);
    procedure ComponentRefUpd;
    procedure UpdateTabList;
    procedure TabChange(Sender: TObject; NewTab: Integer;var AllowChange: Boolean);
    property TextTag:string read FTextTag write FTextTag;
  published
    property VersionInfo:string read FVersionInfo write FTextTag stored false;
    property AutoHide: Boolean read FAutoHide write SetAutoHide default True;
    property AttachSide:TAttachSide read FAttachSide write SetAttachSide default asTop;
    property ButtonWidth:Integer read FButtonWidth write SetButtonWidth default 130;
    property HiddenBarWidth:Integer read FHiddenBarWidth write FHiddenBarWidth default 3;
    property Font;
    property Layout: TButtonLayout read FButtonLayout write SetButtonLayout default blGlyphLeft;
    property Margin:Integer read FMargin write SetMargin default 2;
    property ScrollTime:Integer read FScrollTime write FScrollTime default 400;
    property ShowTitles:Boolean read FShowTitles write SetShowTitles default true;
    property ShowPicture:Boolean read FShowPicture write SetShowPicture default true;
    property Spacing:Integer read FSpacing write SetSpacing default 2;
    {property below must be after Layout and ShowPicture otherwise it wont load this value upon Open}
    property ButtonHeight:Integer read FButtonHeight write SetButtonHeight default 27;
  end;

implementation

uses Consts;

const
      PopItem0     = 'About "%s"';
      PopItem1     = 'Move To';
      PopItem2     = 'Change Title of "%s"';
      PopItem3     = 'Delete "%s"';
      PopItem4     = 'MOVE %s';
      PopItem5     = 'MOVE %s';
      NrOfPopItems = 6;

type
  PNVC = ^TNVC;
  TNVC = record
    Name: string;
    ClassName: string;
    FObject: TComponent;
    Title: string;
    Button:TSpeedButton;
  end;


{ ****  TComponentCollector Component  ****}

procedure TComponentCollector.ShowComponents(Value:Boolean);
var x, y, i:Integer; WinHandle:HWnd; AStr:Array[0..255] of Char; r:TRect; ALongInt:LongInt;
begin
  y:=0;
  if Value then x:=0
  else x:=-10000;

  for i:=0 to FNVCs.Count-1 do
    if Value then
      begin
      ALongInt:=PNVC(FNVCs.Items[i])^.FObject.DesignInfo;
      LongRec(ALongInt).Lo:=x;
      LongRec(ALongInt).Hi:=y;
      PNVC(FNVCs.Items[i])^.FObject.DesignInfo:=ALongInt;
      if (x+27>DForm.Width) then
        begin
        x:=0;
        Inc(y, 30);
        end
      else Inc(x, 30);
      end
    else
      begin
      {$ifdef WIN32}
      SmallInt(LongRec(ALongInt).Lo):=x;
      {$else}
      Integer(LongRec(ALongInt).Lo):=x;
      {$endif}
      LongRec(ALongInt).Hi:=0;
      PNVC(FNVCs.Items[i])^.FObject.DesignInfo:=ALongInt;
      end;

  y:=0;
  if Value then x:=0
  else x:=-10000;

  WinHandle:=GetWindow(DForm.Handle, GW_CHILD);
  WinHandle:=GetWindow(WinHandle, GW_HWNDLAST);
  while (WinHandle<>0) do
    begin
    GetClassName(WinHandle, AStr, 254);
    if (StrComp(AStr, 'TContainer')=0) then
      begin
      {$IFDEF WIN32}
      Windows.GetClientRect(WinHandle, r);
      {$ELSE}
      WinProcs.GetClientRect(WinHandle, r);
      {$ENDIF}
      if (r.Right=28) and (r.Bottom=28) then
        begin
        SetWindowPos(WinHandle, 0, x, y, 0, 0, SWP_NOSIZE or SWP_NOZORDER);
        if (x+27>DForm.Width) then
          begin
          x:=0;
          Inc(y, 30);
          end
        else Inc(x, 30);
        end
      else
        SetWindowPos(WinHandle, 0, x-30, y+30, 0, 0, SWP_NOSIZE or SWP_NOZORDER);
      end;
    WinHandle:=GetWindow(WinHandle, GW_HWNDPREV);
    end;
end;

function TComponentCollector.GetButtonHeight:Integer;
begin
  Result:=20;
  if FShowPicture then
    begin
    Inc(Result, 8);
    if (FButtonLayout=blGlyphTop) or (FButtonLayout=blGlyphBottom) then Inc(Result, 18);
    end;
end;

procedure TComponentCollector.FreeNVC(NVCPointer:Pointer);
begin
  if Assigned(PNVC(NVCPointer)^.Button) then
    PNVC(NVCPointer)^.Button.Free;
  Dispose(PNVC(NVCPointer));
end;

procedure TComponentCollector.FreeNVCs;
var i:Integer;
begin
  if FNVCs.Count>0 then
    for i:=0 to FNVCs.Count-1 do
      if FNVCs.Items[i] <> nil then
        FreeNVC(PNVC(FNVCs.Items[i]));
  FNVCs.Clear;
end;

procedure TComponentCollector.DefineProperties(Filer: TFiler);
begin
  if Filer is TReader then inherited DefineProperties(Filer);
  Filer.DefineProperty('NVCs', ReadData, WriteData, FNVcs.Count > 0);
end;

procedure TComponentCollector.ReadData(Reader: TReader);
var s1, s2, s3:string;
begin
  FreeNVCs;
  Reader.ReadListBegin;
  while not Reader.EndOfList do
    begin
    s1:=Reader.ReadString;
    s2:=Reader.ReadString;
    s3:=Reader.ReadString;
    if (FDesigning) then AddComponent(s1, s2, s3);
    end;
  Reader.ReadListEnd;
end;

procedure TComponentCollector.AddComponent(Name, Title, ClassName:string);
var ComponentRef:PNVC; i, i2:Integer; {$IFDEF WINDOWS}s:string;{$ENDIF}
    TmpInstance:LongInt;
    {$ifdef VER100} LibModule:PLibModule;{$endif}
begin
  {Check our Exclude list}
  if (FExcludeList.IndexOf(Name)<>-1) then Exit;
  {Is it Ok for others to add this component, otherwise add it to our Exclude List}
  for i:=0 to DForm.ComponentCount-1 do
    if (DForm.Components[i] is TComponentCollector) and (DForm.Components[i]<>Self) then
      for i2:=0 to TComponentCollector(DForm.Components[i]).NVCs.Count-1 do
        if (Name=PNVC(TComponentCollector(DForm.Components[i]).NVCs.Items[i2])^.Name) then
          begin
          Self.ExcludeList.Add(Name);
          Exit;
          end;
  {Check that it is not allready added}
  for i:=0 to FNVCs.Count-1 do
    if Name=PNVC(FNVCs.Items[i])^.Name then Exit;
  {Create our component object}
  New(ComponentRef);
  ComponentRef^.Name:=Name;
  ComponentRef^.ClassName:=ClassName;
  ComponentRef^.Title:=Title;
  ComponentRef^.FObject:=nil;
  ComponentRef^.Button:=TSpeedButton.Create(ComponentRef^.FObject);
  ComponentRef^.Button.Visible:=True;
  ComponentRef^.Button.Parent:=Self;
  ComponentRef^.Button.Width:=FButtonWidth;
  ComponentRef^.Button.Height:=FButtonHeight;
  ComponentRef^.Button.OnMouseDown:=BtnClick;
  ComponentRef^.Button.OnDblClick:=BtnDblClick;
  ComponentRef^.Button.GroupIndex:=1;
  ComponentRef^.Button.Layout:=Layout;
  ComponentRef^.Button.Margin:=FMargin;
  ComponentRef^.Button.Spacing:=FSpacing;
  if FShowTitles then
    ComponentRef^.Button.Caption:= ComponentRef^.Title;
  if FShowPicture then
    begin
    DeleteObject(ComponentRef^.Button.Glyph.Handle);
    ComponentRef^.Button.Glyph.Handle:=GetPaletteBitmap(Owner.FindComponent(Name));
    end;
  FNVCs.Add(ComponentRef);

  {Now update all other ComponentCollector exclude lists to contain this name we just added}
  for i:=0 to DForm.ComponentCount-1 do
    if (DForm.Components[i] is TComponentCollector) then
      if (DForm.Components[i]<>Self) then
        with TComponentCollector(DForm.Components[i]) do
          if (ExcludeList.IndexOf(Name)=-1) then
            ExcludeList.Add(Name);
end;

procedure TComponentCollector.WriteData(Writer: TWriter);
var i: Integer;
begin
  Writer.WriteListBegin;
  for i := 0 to FNVCs.Count - 1 do
    begin
    Writer.WriteString(PNVC(FNVCs.Items[i])^.Name);
    Writer.WriteString(PNVC(FNVCs.Items[i])^.Title);
    Writer.WriteString(PNVC(FNVCs.Items[i])^.ClassName);
    end;
  Writer.WriteListEnd;
end;

constructor TComponentCollector.Create(AOwner: TComponent);
var OwnerSize:TRect; PopupItem:TMenuItem; i:Integer;
begin
  {$ifdef WIN32}
  if AOwner.InheritsFrom(TDataModule) then
    begin
    ShowMessage('CompColl can be used in your TDataModule after you have changed '+
                'the Class of the DataModule unit:'+Chr(13)+Chr(13)+
                'FROM THIS:'+Chr(13)+
                '   '+AOwner.ClassName+' = class(TDataModule)'+chr(13)+
                'TO THIS:'+chr(13)+
                '   '+AOwner.ClassName+' = class(TForm)'+chr(13)+Chr(13)+
                'Close and re-open the module then drop a CompColl. Thank you!');
    Abort;
    end;
  {$endif}
  inherited Create(AOwner);
  {$IFDEF WINDOWS}
  FVersionInfo:='16-Bit 2.1';
  {$ELSE}
  {$IFDEF VER100}
  FVersionInfo:='2.1 (Delphi 3)';
  {$ELSE}
  {$IFDEF VER93}
  FVersionInfo:='2.1 (C++Builder)';
  {$ELSE}
  FVersionInfo:='32-Bit 2.1';
  {$ENDIF}
  {$ENDIF}
  {$ENDIF}
  FScrollTime:=400;
  ControlStyle := ControlStyle + [csDesignInteractive, csOpaque];
  OwnerSize:=TForm(Owner).ClientRect;
  DForm:=TForm(Owner);
  Parent:=TWinControl(Owner);
  SetBounds(0, 0, 0, 0);
  FTabSet:=TTabSet.Create(nil);
  FTabSet.Parent:=Self;
  FTabSet.DitherBackground:=False;
  FTabSet.Height:=17;
  FTabSet.Align:=alBottom;
  FTabSet.OnMouseMove:=TabMouseMove;
  FTabSet.OnChange:=TabChange;
  FMargin:=2;
  FSpacing:=2;
  FNVCs := TList.Create;
  FPopup:=TPopupMenu.Create(Self);
  FExcludeList:=TStringList.Create;
  for i:=0 to NrOfPopItems-1 do
    begin
    PopupItem:=TMenuItem.Create(Self);
    PopupItem.Caption := '';
    PopupItem.Tag := (-1)-i;
    PopupItem.OnClick:=PopupVerbMenuClick;
    FPopup.Items.Add(PopupItem);
    end;
  FAutoHide:=True;
  FHidden:=True;
  FAttachSide:=asTop;
  for i:=0 to DForm.ComponentCount-1 do
    if (DForm.Components[i] is TComponentCollector) then
      if (DForm.Components[i]<>Self) then
        if (TComponentCollector(DForm.Components[i]).AttachSide=Self.AttachSide) then
          begin
          FAutoHide:=TComponentCollector(DForm.Components[i]).AutoHide;
          if not FAutoHide then FHidden:=False;
          end;
  FHiddenBarWidth:=3;
  FShowTitles:=True;
  FShowPicture:=True;
  FButtonHeight:=GetButtonHeight;
  FButtonWidth:=130;

  {SetAttachSide(asTop);}
  if (csDesigning in ComponentState) then FDesigning:=True
  else FDesigning:=False;
  FDesigner:=TFormDesigner(DForm.Designer);
  if FDesigning then
    begin
    {Collect;
    SetAttachSide(FAttachSide);}
    end
  else
    Visible:=False;
end;

destructor TComponentCollector.Destroy;
var i, i2:Integer; Moved:Boolean; Found:TComponentCollector; s1, s2, s3:string;
begin
  {KillTimer(Self.Handle, 1);
  KillTimer(Self.Handle, 2);}
  if not (csDestroying in ComponentState) then
    {This happens when you delete this component at design-time, but not at form destroy}
    begin
    Found:=nil;
    Moved:=False;
    for i:=0 to DForm.ComponentCount-1 do
      if (DForm.Components[i] is TComponentCollector) and (DForm.Components[i]<>Self) then
        begin {Move the collected components to the other}
        for i2:=Self.FNVCs.Count-1 downto 0 do
          with TComponentCollector(DForm.Components[i]) do
            begin
            s1:=PNVC(Self.FNVCs.Items[i2])^.Name;
            s2:=PNVC(Self.FNVCs.Items[i2])^.Title;
            s3:=PNVC(Self.FNVCs.Items[i2])^.ClassName;
            Self.FreeNVC(Self.FNVCs.Items[i2]);
            Self.FNVCs.Delete(i2);
            Self.FNVCs.Pack;
            if (ExcludeList.IndexOf(s1)<>-1) then
              ExcludeList.Delete(ExcludeList.IndexOf(s1));
            AddComponent(s1, s2, s3);
            Moved:=True;
            end;
        Found:=TComponentCollector(DForm.Components[i]);
        if Moved then
          ShowMessage('Collected components of "'+Self.Name+'" has been moved to "'+
                      DForm.Components[i].Name+'", due to Delete action');
        Break;
        end;
    if not Assigned(Found) then ShowComponents(True);
    end;
  FTabSet.Free;
  FExcludeList.Free;
  FPopup.Free;
  if Assigned(FSelectedEditor) then FSelectedEditor.Free;
  FreeNVCs;
  FNVCs.Free;
  inherited Destroy;
end;

procedure TComponentCollector.CreateParams(var Params: TCreateParams);
const BorderStyles: array[TBorderStyle] of Longint = (0, WS_BORDER);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style or BorderStyles[bsNone];
end;

procedure TComponentCollector.Loaded;
begin
  Inherited Loaded;
  if (FDesigning) then
    begin
    if (FAttachSide=asLeft) then Left:=3-Width
    else if(FAttachSide=asTop) then Top:=3-Height;
    {$IFDEF TRIAL}
    ShowMessage('Trial reminder for '+Name+' !'+Chr(13)+
                'Get rid of this annoying message. Buy the full version with source '+
                'for only US$19. Please see COMPCOLL.TXT or visit http://www.lingscape.com/');
    {$ENDIF}
    end;
end;

procedure TComponentCollector.ComponentRefUpd;
var i, xTop, xLeft:Integer;
begin
  for i:=FNVCs.Count -1 downto 0 do
    begin
    if not Assigned(PNVC(FNVCs.Items[i])^.FObject) then
      PNVC(FNVCs.Items[i])^.FObject:=DForm.FindComponent(PNVC(FNVCs.Items[i])^.Name);
    if not Assigned(PNVC(FNVCs.Items[i])^.FObject) then
      begin
      FreeNVC(PNVC(FNVCs.Items[i]));
      FNVCs.Items[i]:=nil;
      FNVCs.Pack;
      end;
    end;
  xTop:=2; xLeft:=2;
  for i:=0 to FNVCs.Count -1 do
    begin
    if (TComponent(PNVC(FNVCs.Items[i])^.FObject).Name<>PNVC(FNVCs.Items[i])^.Name) then
      begin
      if (PNVC(FNVCs.Items[i])^.Title=PNVC(FNVCs.Items[i])^.Name) then
        PNVC(FNVCs.Items[i])^.Title:=TComponent(PNVC(FNVCs.Items[i])^.FObject).Name;
      PNVC(FNVCs.Items[i])^.Name:=TComponent(PNVC(FNVCs.Items[i])^.FObject).Name;
      end;
    if (FShowPicture and (PNVC(FNVCs.Items[i])^.Button.Glyph.Handle=0)) then
      PNVC(FNVCs.Items[i])^.Button.Glyph.Handle:=GetPaletteBitmap(PNVC(FNVCs.Items[i])^.FObject);
    if FShowTitles then
      PNVC(FNVCs.Items[i])^.Button.Caption:=PNVC(FNVCs.Items[i])^.Title
    else
      PNVC(FNVCs.Items[i])^.Button.Caption:='';
    PNVC(FNVCs.Items[i])^.Button.Top:=xTop;
    PNVC(FNVCs.Items[i])^.Button.Left:=xLeft;
    PNVC(FNVCs.Items[i])^.Button.Width:=FButtonWidth;
    PNVC(FNVCs.Items[i])^.Button.Height:=FButtonHeight;
    case FAttachSide of
      asLeft : begin
               Inc(xTop, FButtonHeight);
               if (xTop+FButtonHeight+FTabSet.Height>DForm.ClientHeight) then
                 begin
                 xTop:=2;
                 Inc(xLeft, FButtonWidth);
                 end;
               end;
      asTop  : begin
               Inc(xLeft, FButtonWidth);
               if (xLeft+FButtonWidth>DForm.ClientWidth) then
                 begin
                 xLeft:=2;
                 Inc(xTop, FButtonHeight);
                 end;
               end;
      asNone : begin
               Inc(xLeft, FButtonWidth);
               if (xLeft+FButtonWidth>ClientWidth) then
                 begin
                 xLeft:=2;
                 Inc(xTop, FButtonHeight);
                 end;
               end;
      end;
    end;
end;

procedure TComponentCollector.Paint;
var R:TRect;
begin
  with Canvas do
    begin
    Brush.Color := clBtnFace;
    R:=ClientRect;
    Canvas.FillRect(R);
    Font := Self.Font;
    {$IFDEF WIN32}
    DrawEdge(Canvas.Handle, R, BDR_RAISEDINNER, BF_TOPLEFT);
    DrawEdge(Canvas.Handle, R, BDR_RAISEDINNER, BF_BOTTOMRight);
    {$ELSE}
    DrawFocusRect(r);
    {$ENDIF}
    end;
end;

procedure TComponentCollector.Collect;
var Index:Integer;
begin
  if (csLoading in ComponentState) or not (FDesigning) then Exit;
  BringToFront;
  {Updates the Exclude list, in case it contains components which no longer exists}
  for Index:=FExcludeList.Count-1 downto 0 do
    if (FindComponent(FExcludeList[Index])<>nil) then
      FExcludeList.Delete(Index);
  {Remove components that doesn't exist on the form any longer}
  with DForm do
    for Index:=FNVCs.Count-1 downto 0 do
      if (FindComponent(PNVC(FNVCs.Items[Index])^.Name)=nil) then
        begin
        FreeNVC(FNVCs.Items[Index]);
        FNVCs.Delete(Index);
        FNVCs.Pack;
        end;
  {Collect components}
  for Index:=0 to DForm.ComponentCount-1 do
    if (DForm.Components[Index].Name<>'') then
      with DForm do
        if CanWeCollect(Components[Index]) then
          AddComponent(Components[Index].Name, Components[Index].Name, Components[Index].ClassName);
  ComponentRefUpd;
  ShowComponents(False);
  UpdateTabList;
end;

function TComponentCollector.CanWeCollect(AComp:TComponent):Boolean;
const InvalidCollect: array [1..4] of string = (
                      'TField',
                      'TControl',
                      'TComponentCollector',
                      'TMenuItem');
var i:integer;
  function InheritsFromClassName(ClassName:string):Boolean;
  var AClass:TClass;
  begin
    Result:=False;
    AClass:=AComp.ClassType;
    while AClass<>nil do
      begin
      if (AClass.ClassName=ClassName) then
        begin
        Result:=True;
        Break;
        end;
      AClass:=AClass.ClassParent;
      end;
  end;
begin
  Result:=True;
  for i:=1 to 4 do
    if InheritsFromClassName(InvalidCollect[i]) then
      begin
      Result:=False;
      Break;
      end;
end;

procedure TComponentCollector.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if (AComponent=nil) then Exit;
  if (AComponent.Name=Self.Name) then Exit;
  if ([csDesigning]=ComponentState) then
    PostMessage(Self.Handle, CM_COLLECT, 0, LongInt(Self));
  Inherited Notification(AComponent, Operation);
end;

procedure TComponentCollector.SetShowTitles(Value:Boolean);
var i:Integer;
begin
  if Value<>FShowTitles then
    begin
    for i:=0 to FNVCs.Count -1 do
      if Value then
        TSpeedButton(PNVC(FNVCs.Items[i])^.Button).Caption:=PNVC(FNVCs.Items[i])^.Title
      else
        TSpeedButton(PNVC(FNVCs.Items[i])^.Button).Caption:='';
    FShowTitles:=Value;
    end;
end;

procedure TComponentCollector.SetMargin(Value:Integer);
var i:Integer;
begin
  if Value<>FMargin then
    begin
    for i:=0 to FNVCs.Count -1 do
      TSpeedButton(PNVC(FNVCs.Items[i])^.Button).Margin:=Value;
    FMargin:=Value;
    end;
end;

procedure TComponentCollector.SetSpacing(Value:Integer);
var i:Integer;
begin
  if Value<>FSpacing then
    begin
    for i:=0 to FNVCs.Count -1 do
      TSpeedButton(PNVC(FNVCs.Items[i])^.Button).Spacing:=Value;
    FSpacing:=Value;
    end;
end;

procedure TComponentCollector.SetShowPicture(Value:Boolean);
var i:Integer; s:string; TmpInstance:LongInt;
begin
  if Value<>FShowPicture then
    begin
    for i:=0 to FNVCs.Count -1 do
      if Value and Assigned(PNVC(FNVCs.Items[i])^.FObject) then
        begin
        DeleteObject(PNVC(FNVCs.Items[i])^.Button.Glyph.Handle);
        PNVC(FNVCs.Items[i])^.Button.Glyph.Handle:=GetPaletteBitmap(PNVC(FNVCs.Items[i])^.FObject);
        end
      else
        begin
        DeleteObject(PNVC(FNVCs.Items[i])^.Button.Glyph.Handle);
        PNVC(FNVCs.Items[i])^.Button.Glyph.Handle:=0;
        end;
    FShowPicture:=Value;
    ButtonHeight:=GetButtonHeight;
    end;
end;

function TComponentCollector.GetPaletteBitmap(AComp:TComponent):THandle;
var AClass:TClass; s:string;
    {$ifdef VER100} LibModule:PLibModule; {$endif}
begin
  Result:=0;
  if (AComp=nil) then Exit;
  AClass:=AComp.ClassType;
  while (AClass<>nil) do
    begin
    s:=AClass.ClassName+Chr(0);
    {$ifdef VER100}
    LibModule := LibModuleList;
    while LibModule <> nil do
      with LibModule^ do
        begin
        Result := LoadBitmap(ResInstance, @s[1]);
        if (Result <> 0) then Break
        else LibModule:=Next;
        end;
    {$else}
    Result:=LoadBitmap(HInstance, @s[1]);
    {$endif}
    if (Result=0) then AClass:=AClass.ClassParent
    else Break;
    end;
end;

procedure TComponentCollector.SetButtonLayout(Value:TButtonLayout);
var i:Integer;
begin
  if Value<>FButtonLayout then
    begin
    for i:=0 to FNVCs.Count -1 do
      TSpeedButton(PNVC(FNVCs.Items[i])^.Button).Layout:=Value;
    FButtonLayout:=Value;
    ButtonHeight:=GetButtonHeight;
    end;
end;

procedure TComponentCollector.SetButtonHeight(Value:Integer);
begin
  if (Value<>FButtonHeight) then
    begin
    FButtonHeight:=Value;
    ComponentRefUpd;
    Paint;
    end;
end;

procedure TComponentCollector.SetButtonWidth(Value:Integer);
begin
  if (Value<>FButtonWidth) then
    begin
    FButtonWidth:=Value;
    ComponentRefUpd;
    Paint;
    end;
end;

procedure TComponentCollector.SetAttachSide(Value:TAttachSide);
var xLeft, xTop : Integer;
begin
  xLeft:=0; xTop:=0;
  case Value of
    asLeft   : if FHidden then xLeft:=HiddenBarWidth-GetBarWidth;
    asTop    : if FHidden then xTop:=HiddenBarWidth-GetBarHeight;
    end;
  if (FAttachSide<>Value) then
    begin
    if (Value=asNone) then ShowBar;
    FAttachSide:=Value;
    if ([csDesigning] = ComponentState) then UpdateTabList;
    end;
  case Value of
    asLeft   : if not ((Left=xLeft) and (Top=xTop) and (Width=GetBarWidth) and (Height=GetBarHeight)) then
                 SetBounds(xLeft, xTop, GetBarWidth, GetBarHeight);
    asTop    : if not ((Left=xLeft) and (Top=xTop) and (Width=GetBarWidth) and (Height=GetBarHeight)) then
                 SetBounds(xLeft, xTop, GetBarWidth, GetBarHeight);
    end;
  ComponentRefUpd;
end;

procedure TComponentCollector.SetAutoHide(Value:Boolean);
var i:Integer;
begin
  if (Value<>FAutoHide) then
    begin
    for i:=0 to DForm.ComponentCount -1 do
      if (DForm.Components[i] is TComponentCollector) then
        SendMessage(TComponentCollector(DForm.Components[i]).Handle,
                    CM_AUTOHIDE, LongInt(Value), LongInt(Self));
    if Value then
      begin
      FHidden:=False;
      HideBar;
      end
    else ShowBar;

    FAutoHide:=Value;
    end;
end;

procedure TComponentCollector.UpdateTabList;
var i, i2:Integer;
begin
  FTabSet.Tabs.Clear;
  for i:=0 to DForm.ComponentCount-1 do
    if (DForm.Components[i] is TComponentCollector) and
        (TComponentCollector(DForm.Components[i]).AttachSide=FAttachSide) then
      begin
      i2:=FTabSet.Tabs.Add(DForm.Components[i].Name);
      if (DForm.Components[i] = Self) then FTabSet.TabIndex:=i2;
      end;
end;

procedure TComponentCollector.PopupVerbMenuClick(Sender: TObject);
var s, s1, s2, s3:string; i, Selected: Integer; SelList:TComponentList; Done:Boolean;
    {$IFDEF WINDOWS}Editor:TComponentEditor;{$ENDIF}
begin
  if not FDesigning then Exit;
  {$IFDEF WIN32}
  SelList:=TComponentList.Create;
  try
    Selected:=-1;
    FDesigner.GetSelections(SelList);
    for i:=0 to FNVCs.Count-1 do
      if (PNVC(FNVCs.Items[i])^.FObject=SelList.Items[0]) then
        begin
        Selected:=i;
        Break;
        end;
  finally
    SelList.Free;
  end;
  {$ELSE}
  Editor:=GetComponentEditor(Self, nil);
  try
    for i:=0 to FNVCs.Count-1 do
      if (PNVC(FNVCs.Items[i])^.Name=Editor.GetVerb(100)) then
        begin
        Selected:=i;
        Break;
        end;
  finally
    Editor.Free;
  end;
  {$ENDIF}
  Done:=True;
  case TMenuItem(Sender).Tag of
    -6     : begin {MOVE DOWN}
             FNVCs.Exchange(Selected, Selected+1);
             end;
    -5     : begin {MOVE UP}
             FNVCs.Exchange(Selected, Selected-1);
             end;
    -4     : begin {DELETE}
             DForm.Perform(WM_KEYDOWN, VK_DELETE, 0);
             {Collect;}
             end;
    -3     : repeat {CHANGE TITLE}
               s:=PNVC(FNVCs.Items[Selected])^.Title;
               InputQuery('Title of '+PNVC(FNVCs.Items[Selected])^.Name, 'Title', s);
               if (s=PNVC(FNVCs.Items[Selected])^.Title) then Break;
               Done:=True;
               for i:=0 to FNVCs.Count-1 do
                 if (PNVC(FNVCs.Items[i])^.Title=s) then
                   begin
                   MessageDlg(s+' is allready assigned to '+PNVC(FNVCs.Items[i])^.Name+
                              ', please enter another title', mtError, [mbOK], 0);
                   s:=PNVC(FNVCs.Items[Selected])^.Title;
                   Done:=False;
                   break;
                   end;
               if (s<>PNVC(FNVCs.Items[Selected])^.Title) then
                 begin
                 FDesigner.Modified;
                 PNVC(FNVCs.Items[Selected])^.Title:=s;
                 end;
             until Done;
    -2     : ;
    -199..-100 : begin
             s1:=PNVC(FNVCs.Items[Selected])^.Name;
             s2:=PNVC(FNVCs.Items[Selected])^.Title;
             s3:=PNVC(FNVCs.Items[Selected])^.ClassName;
             FreeNVC(FNVCs.Items[Selected]);
             FNVCs.Delete(Selected);
             FNVCs.Pack;
             FExcludeList.Add(s1);
             ComponentRefUpd;
             with TComponentCollector(DForm.FindComponent(TMenuItem(Sender).Caption)) do
               begin
               if (ExcludeList.IndexOf(s1)<>-1) then
                 ExcludeList.Delete(ExcludeList.IndexOf(s1));
               AddComponent(s1, s2, s3);
               ComponentRefUpd;
               end;
             end;
    -1     : begin
             ShowMessage('Title: '+PNVC(FNVCs.Items[Selected])^.Title+Chr(13)+
                         'Name:  '+PNVC(FNVCs.Items[Selected])^.Name+Chr(13)+
                         'Class: '+PNVC(FNVCs.Items[Selected])^.ClassName+Chr(13)+
                         'Inherited Class: '+PNVC(FNVCs.Items[Selected])^.FObject.ClassParent.ClassName);
             end;
    0..999 : FSelectedEditor.ExecuteVerb(TMenuItem(Sender).Tag);
  end;
end;

procedure TComponentCollector.BtnClick(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var CmpList:TComponentList; i, s:Integer; Selected:TComponent;
    cp, sp:TPoint; PopupItem:TMenuItem; Str:String; {$IFDEF WINDOWS}Editor:TComponentEditor;{$ENDIF}
begin
  if not FDesigning then Exit;
  if FNVCs.Count=0 then Exit;
  s:=0;
  for i:=0 to FNVCs.Count-1 do
    if Sender=PNVC(FNVCs.Items[i])^.Button then
      begin
      s:=i;
      break;
      end;
  Selected:=PNVC(FNVCs.Items[s])^.FObject;
  CmpList:=TComponentList.Create;
  try
    CmpList.Add(Selected);
    if Assigned(FSelectedEditor) then FSelectedEditor.Free;
    FSelectedEditor:=GetComponentEditor(Selected, FDesigner);
    {$IFDEF WIN32}
    FDesigner.SelectComponent(Selected);
    {$ELSE}
    Editor:=GetComponentEditor(Self, nil);
    try
      FTextTag:=Selected.Name;
      Editor.GetVerb(101);
    finally
      Editor.Free;
    end;
    {$ENDIF}
    if Button=mbRight then
      begin
      while (FPopup.Items.Count>NrOfPopItems) do
        FPopup.Items.Remove(FPopup.Items[0]);
      for i:=0 to FPopup.Items.Count-1 do
        case i of
          0 : FPopup.Items[i].Caption:=Format(PopItem0, [PNVC(FNVCs.Items[s])^.Title]);
          1 : begin
              FPopup.Items[i].OnClick:=nil;
              FPopup.Items[i].Caption:=PopItem1;
              InsertGroupList(FPopup.Items[i]);
              if (TMenuItem(FPopup.Items[i]).Count=0) then
                FPopup.Items[i].Enabled:=False
              else
                FPopup.Items[i].Enabled:=True;
              end;
          2 : FPopup.Items[i].Caption:=Format(PopItem2, [PNVC(FNVCs.Items[s])^.Name]);
          3 : FPopup.Items[i].Caption:=Format(PopItem3, [PNVC(FNVCs.Items[s])^.Name]);
          4 : begin
              if (FAttachSide=asLeft) then Str:='UP'
              else Str:='LEFT';
              FPopup.Items[i].Caption:=Format(PopItem4, [Str]);
              if (s=0) then FPopup.Items[i].Enabled:=False
              else FPopup.Items[i].Enabled:=True;
              end;
          5 : begin
              if (FAttachSide=asLeft) then Str:='DOWN'
              else Str:='RIGHT';
              FPopup.Items[i].Caption:=Format(PopItem5, [Str]);
              if (s=FNVCs.Count-1) then FPopup.Items[i].Enabled:=False
              else FPopup.Items[i].Enabled:=True;
              end;
        end;
      if FSelectedEditor.GetVerbCount>0 then
        begin
        PopupItem:=TMenuItem.Create(Self);
        PopupItem.Caption := '-';
        PopupItem.Tag := 0;
        FPopup.Items.Insert(0, PopupItem);
        end;
      for i:=0 to FSelectedEditor.GetVerbCount-1 do
        begin
        PopupItem:=TMenuItem.Create(Self);
        PopupItem.Caption := FSelectedEditor.GetVerb(i);
        PopupItem.Tag := i;
        PopupItem.OnClick:=PopupVerbMenuClick;
        FPopup.Items.Insert(i, PopupItem);
        end;
      TSpeedButton(Sender).Down:=True;
      cp.x:=x;
      cp.y:=y;
      sp:=TSpeedButton(Sender).ClientToScreen(cp);
      FPopup.Popup(sp.x, sp.y);
      end;
  finally
    CmpList.Free;
  end;
end;

procedure TComponentCollector.InsertGroupList(PopItem: TMenuItem);
var i, Cnt:Integer; SubMenu:TMenuItem;
begin
  Cnt:=0;
  for i:=PopItem.Count-1 downto 0 do
    PopItem.Remove(PopItem.Items[i]);
  for i:=0 to DForm.ComponentCount-1 do
    if (DForm.Components[i] is TComponentCollector) then
      if (DForm.Components[i]<>Self) then
        begin
        SubMenu:=TMenuItem.Create(Self);
        SubMenu.Caption := DForm.Components[i].Name;
        SubMenu.Tag := (-100)-Cnt;
        SubMenu.OnClick:=PopupVerbMenuClick;
        PopItem.Add(SubMenu);
        Inc(Cnt);
        end;
end;

procedure TComponentCollector.BtnDblClick(Sender:TObject);
begin
  if not FDesigning then Exit;
  FSelectedEditor.Edit;
end;

procedure TComponentCollector.TabChange(Sender: TObject; NewTab: Integer;var AllowChange: Boolean);
var AComp:TComponent; i:Integer; {$IFDEF WINDOWS}Editor:TComponentEditor;{$ENDIF}
begin
  if not FDesigning then Exit;
  if (FTabSet.TabIndex<>-1) then
    begin
    AComp:=DForm.FindComponent(FTabSet.Tabs[NewTab]);
    {KillTimer(Self.Handle, FTimerID);}
    HideBar;
    TComponentCollector(AComp).ShowBar;
    {$IFDEF WIN32}
    FDesigner.SelectComponent(AComp);
    {$ELSE}
    Editor:=GetComponentEditor(Self, nil);
    try
      FTextTag:=AComp.Name;
      Editor.GetVerb(101);
    finally
      Editor.Free;
    end;
    {$ENDIF}
    for i:=0 to FNVCs.Count-1 do
      if (PNVC(FNVCs.Items[i])^.Button.Down) then
        begin
        PNVC(FNVCs.Items[i])^.Button.AllowAllUp:=True;
        PNVC(FNVCs.Items[i])^.Button.Down:=False;
        PNVC(FNVCs.Items[i])^.Button.AllowAllUp:=False;
        Break;
        end;
    AllowChange:=False;
    end;
end;

procedure TComponentCollector.TabMouseMove(Sender:TObject; Shift: TShiftState; X, Y: Integer);
begin
  MouseMove(Shift, x, y);
end;

procedure TComponentCollector.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseMove(Shift, X, Y);
  if (Parent<>Owner) then
    {this if statement runs when this component is dropped on a TPanel or on another
     component which can store own components.}
    begin
    Parent:=TWinControl(Owner);
    SetAttachSide(FAttachSide);
    Collect;
    end;
  if (FAttachSide=asTop) and (y>30) then Exit;
  if (FAttachSide=asLeft) and ((x<Width-30) or (x>Width)) then Exit;
  if (FHidden and (Top<10)) then ShowBar;
end;

function TComponentCollector.GetBarHeight:Integer;
var NrPerWidth, x:Integer;
begin
  Result:=0;
  case FAttachSide of
    asLeft   : begin
               Result:=DForm.ClientHeight;
               for x:=0 to DForm.ComponentCount-1 do
                 if (DForm.Components[x].InheritsFrom(TControl)) then
                   if (DForm.Components[x]<>Self) then
                     if (TControl(DForm.Components[x]).Align=alBottom) then
                       begin
                       Dec(Result, TControl(DForm.Components[x]).Height);
                       end;
               if (Result<0) then Result:=0;
               end;
    asTop    : begin
               if (DForm.ClientWidth>FButtonWidth) then
                 begin
                 NrPerWidth:=(DForm.ClientWidth div FButtonWidth);
                 x:=FNVCs.Count div NrPerWidth;
                 if (FNVCs.Count/NrPerWidth)>(FNVCs.Count div NrPerWidth) then Inc(x);
                 end
               else x:=1;
               if (x=0) then x:=1;
               Result:=x*ButtonHeight+10;
               Inc(Result, FTabSet.Height);
               end;
    asNone   : Result:=ClientHeight;
  end;
end;

function TComponentCollector.GetBarWidth:Integer;
var NrPerHeight, x, i:Integer;
begin
  Result:=0;
  case FAttachSide of
    asLeft   : begin
               if (DForm.ClientHeight>FButtonHeight) then
                 begin
                 NrPerHeight:=((DForm.ClientHeight-(FTabSet.Height+2)) div FButtonHeight);
                 x:=FNVCs.Count div NrPerHeight;
                 if (FNVCs.Count/NrPerHeight)>(FNVCs.Count div NrPerHeight) then Inc(x);
                 end
               else x:=1;
               if (x=0) then x:=1;
               {For loop below is to increase the size of the width if there are other collectors
               around, since the name of the tab bar otherwise wont fit}
               for i:=0 to DForm.ComponentCount-1 do
                 if (DForm.Components[i] is TComponentCollector) and (DForm.Components[i]<>Self) and
                    (TComponentCollector(DForm.Components[i]).AttachSide=asLeft) then
                   begin
                   if x=1 then Inc(x);
                   end;
               Result:=x*ButtonWidth+10;
               end;
    asTop    : Result:=DForm.ClientWidth;
    asNone   : Result:=ClientWidth;
  end;
end;

procedure TComponentCollector.HideBar;
begin
  if not FHidden and FAutoHide then
    begin
    case AttachSide of
      asLeft   : ScrollBar((0-Width)+HiddenBarWidth);
      asTop    : ScrollBar((0-Height)+HiddenBarWidth);
    end;
    FHidden:=True;
    end;
end;

procedure TComponentCollector.ShowBar;
begin
  UpdateTabList;
  BringToFront;
  if FHidden then
    begin
    case AttachSide of
      asLeft   : ScrollBar(Width-HiddenBarWidth);
      asTop    : ScrollBar(Height-HiddenBarWidth);
    end;
    FHidden:=False;
    end;
end;

procedure TComponentCollector.ScrollBar(Points:Integer);
var t0, t, OrgLeft, OrgTop, ScrollPoints:LongInt;
begin
  OrgLeft:=Left;
  OrgTop:=Top;
  t0:=GetTickCount;
  t:=t0;
  while (t < (t0 + FScrollTime)) do
    begin
    t:=GetTickCount;
    ScrollPoints:=Round(Points*((t - t0) / FScrollTime));
    case AttachSide of
      asLeft  : if (OrgLeft+ScrollPoints<=0) then Left:=OrgLeft+ScrollPoints;
      asTop   : if (OrgTop+ScrollPoints<=0) then Top:=OrgTop+ScrollPoints;
      asNone   : ;
      end;
    UpdateWindow(Self.Handle);
    UpdateWindow(DForm.Handle);
    end;
  case AttachSide of
    asLeft  : SetBounds(OrgLeft+Points, Top, Width, Height);
    asTop   : SetBounds(Left, OrgTop+Points, Width, Height);
    asNone   : ;
    end;
end;

procedure TComponentCollector.CMCollect(var Msg: TMessage);
begin
  if (TComponent(Msg.lParam)=Self) then
    begin
    Application.ProcessMessages;
    Collect;
    end;
end;

procedure TComponentCollector.CMAutoHide(var Msg: TMessage);
var OldTime:Integer;
begin
  if (TComponent(Msg.lParam)=Self) then Exit;
  if (Msg.wParam=0) then
    begin
    FAutoHide:=False;
    OldTime:=FScrollTime;
    FScrollTime:=0;
    ShowBar;
    FScrollTime:=OldTime;
    end
  else if (Msg.wParam=1) then
    begin
    FAutoHide:=True;
    OldTime:=FScrollTime;
    FScrollTime:=0;
    HideBar;
    FScrollTime:=OldTime;
    end;
end;

procedure TComponentCollector.WMTimer(var Msg: TWMTimer);
var r:TRect; pt:TPoint; SelList:TComponentList; i, i2:Integer; Found:Boolean;
    {$IFDEF WINDOWS}Editor:TComponentEditor;{$ENDIF}
begin
  {if TimerID=2 it was just created on the form}
  if (Msg.TimerID=2) then
    begin
    KillTimer(Self.Handle, 2);
    Collect;
    end;
  GetCursorPos(pt);
  GetWindowRect(Self.Handle, r);
  InflateRect(r, 40, 40);
  if not PtInRect(r, pt) then
    if not FHidden then
      begin
      SelList:=TComponentList.Create;
      try
        {$IFDEF WIN32}
        FDesigner.GetSelections(SelList);
        {$ELSE}
        Editor:=GetComponentEditor(Self, nil);
        try
          SelList.Add(TForm(Owner).FindComponent(Editor.GetVerb(100)));
        finally
          Editor.Free;
        end;
        {$ENDIF}
        Found:=False;
        for i:=0 to SelList.Count-1 do
          begin
          if (SelList.Items[i]=Self) then
            begin
            Found:=True;
            Break;
            end;
          if Found then Break;
          for i2:=0 to FNVCs.Count-1 do
            if (PNVC(FNVCs.Items[i2])^.FObject=SelList.Items[i]) then
              begin
              Found:=True;
              Break;
              end;
          if Found then Break;
          end;
        if not Found then HideBar;
      finally
        SelList.Free;
      end;
      end;
  SetAttachSide(FAttachSide);
  inherited;
end;

procedure TComponentCollector.WMCreate(var Msg: TWMCreate);
begin
  FTimerID:=SetTimer(Self.Handle, 2, 1, nil);
  FTimerID:=SetTimer(Self.Handle, 1, 200, nil);
end;

procedure TComponentCollector.WMDestroy(var Msg: TWMDestroy);
begin
  KillTimer(Self.Handle, 2);
  KillTimer(Self.Handle, 1);
end;

procedure Register;
begin
  RegisterComponents('Lingscape', [TComponentCollector]);
end;

end.

